/*
 * Copyright 2013 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.launcher.ui;

import java.io.File;

import info.msxlaunchers.openmsx.common.OSUtils;
import info.msxlaunchers.openmsx.extension.ExtensionListerModule;
import info.msxlaunchers.openmsx.game.repository.RepositoryDataModule;
import info.msxlaunchers.openmsx.game.scan.ScannerModule;
import info.msxlaunchers.openmsx.launcher.extra.ExtraDataModule;
import info.msxlaunchers.openmsx.launcher.persistence.filter.FilterPersisterModule;
import info.msxlaunchers.openmsx.launcher.persistence.game.GamePersisterModule;
import info.msxlaunchers.openmsx.launcher.persistence.settings.SettingsPersisterModule;
import info.msxlaunchers.openmsx.launcher.starter.StarterModule;
import info.msxlaunchers.openmsx.launcher.ui.presenter.MainPresenter;
import info.msxlaunchers.openmsx.launcher.ui.presenter.PresenterModule;
import info.msxlaunchers.openmsx.launcher.ui.view.ViewModule;
import info.msxlaunchers.openmsx.machine.MachineListerModule;
import info.msxlaunchers.platform.PlatformModule;

import com.google.inject.Guice;
import com.google.inject.Injector;

/**
 * Main class that starts the application
 * 
 * @since v1.2
 * @author Sam Elsharif
 *
 */
public class Launcher
{
	private final static String APP_NAME = "openMSX Launcher";

	private final static String APP_DIR;
	private final static String USER_HOME_DIR;
	private final static String USER_DATA_DIR;
	private final static String EXTRA_DATA_DIR;
	private final static String GENERATION_MSX_URL;

	static
	{
		APP_DIR = System.getProperty( "user.dir" );
		USER_HOME_DIR = System.getProperty( "user.home" );
		GENERATION_MSX_URL = "http://www.generation-msx.nl/msxdb/softwareinfo/";

		if( OSUtils.isWindows() )
		{
			USER_DATA_DIR = APP_DIR;
			EXTRA_DATA_DIR = APP_DIR;
		}
		else if( OSUtils.isMac() )
		{
			USER_DATA_DIR = USER_HOME_DIR + "/Library/Application Support/" + APP_NAME;
			EXTRA_DATA_DIR = APP_DIR + "/" + APP_NAME + ".app/share";
		}
		else if( OSUtils.isLinux() || OSUtils.isBSD() )
		{
			USER_DATA_DIR = USER_HOME_DIR + "/" + APP_NAME;
			EXTRA_DATA_DIR = APP_DIR;
		}
		else
		{
			//this is not supported for now, but continue anyway
			USER_DATA_DIR = APP_DIR;
			EXTRA_DATA_DIR = APP_DIR;
		}
	}

	public static void main( String [] args )
	{
		//before anything, make sure user data directories exist
		createUserDataDirectoryIfNecessary();

		Injector injector = Guice.createInjector(
				new AppModule( USER_DATA_DIR, EXTRA_DATA_DIR, GENERATION_MSX_URL ),
				new SettingsPersisterModule(),
	    		new GamePersisterModule(),
	    		new StarterModule(),
	    		new PlatformModule(),
	    		new ViewModule(),
	    		new PresenterModule(),
	    		new ScannerModule(),
	    		new RepositoryDataModule(),
	    		new MachineListerModule(),
	    		new ExtensionListerModule(),
	    		new ExtraDataModule(),
	    		new FilterPersisterModule()
				);

		MainPresenter launcher = injector.getInstance( MainPresenter.class );

		launcher.start();
	}

	private static void createUserDataDirectoryIfNecessary()
	{
		File userDataDirectory = new File( USER_DATA_DIR );

		if( !userDataDirectory.exists() )
		{
			userDataDirectory.mkdir();
		}
	}
}
